#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: vz_backup_parse.pl, v1.1.0.0 Tue 11 Mar 2008 02:42:51 PM CET mstauber Exp $
# Copyright 2006-2012 Solarspeed Ltd. All rights reserved.

# This constructor parses /etc/vz/aconf/vps-mn-backup.conf and pushes the info into CCE.
# This is especially important for clusters on fail overs, so that the backup settings
# are inherited on the new primary after a fail-over.

$whatami = "constructor";

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

$DEBUG = 0;
if ($DEBUG)
{
        use Sys::Syslog qw( :DEFAULT setlogsock);
}


# Location of the config file:
$vserver_cronConfig = "/etc/vz/aconf/vps-mn-backup.conf";

# Config file present?
if (-e $vserver_cronConfig) {
        # Parse config file and shove their data into CCE:
        &main;
        # Update backup cronjob
        &finalize;
}
else {
        if (!-d "/vz/private") {
                # Disable backup on the secondary:
                &disable_backup;
                # Update backup cronjob
                &finalize;
        }
}

exit 0;

##
# Subs:
##

sub main {

    # Pull the config file into a Matrix:
    #
    # I LOVE matrix stuff! If there is a sledload of data to parse, 
    # then there is hardly a more efficient way to do so.
    # A legend of what colums the matrix has is at the end of the code.

    open (F, $vserver_cronConfig) || die "Could not open $vserver_cronConfig: $!";
    $a = 0;                             # Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    while ($line = <F>) {
        chomp($line);
        next if $line =~ /^\s*$/;               # skip blank lines

        # Special case for first item:
        if ($line =~ /^MN_LocalBackupEnabled(.*)/) {
            $matrix_name = 'CONFIG';
            my (@row) = split (/\s+/, $line);
            push (@{$matrix_name}, \@row);
            if (($CONFIG[$a][0] ne "") && ($CONFIG[$a][1] ne "")) {
                &feedthemonster;
            }
        }

        if ($line =~ /^([A-Za-z_]\w*)/) {
            #$matrix_name = $1;
            $matrix_name = 'CONFIG';
            my (@row) = split (/\s+/, $line);
            push (@{$matrix_name}, \@row);
            #&debug_msg("Processing: $line | $row[0] : $row[1] | A0: $CONFIG[$a][0] - A1: $CONFIG[$a][1] \n");
            if (($CONFIG[$a][0] ne "") && ($CONFIG[$a][1] ne "")) {
                &feedthemonster;
            }
        }
        else {
                &debug_msg("Skipping: $line \n");
        }
        $a++;
    }
    close(F);

}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
    # This is where we actually populate CCE with the gathered info:
    @oids = $cce->find('VserverMasterNode');
    if ($#oids < 0) {
        # Object not yet in CCE. Creating new one.
        ($ok) = $cce->create('VserverMasterNode', {
            $CONFIG[$a][0] => $CONFIG[$a][1]
        });
    }
    else {
        # Object already present in CCE. Updating it
        ($sys_oid) = $cce->find('VserverMasterNode');
        ($ok, $sys) = $cce->get($sys_oid);

        ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            $CONFIG[$a][0] => $CONFIG[$a][1]
        });
    }

}

sub finalize {
    # Triger the updating of the vz-backup cronjob
    ($sys_oid) = $cce->find('VserverMasterNode');
    ($ok, $sys) = $cce->get($sys_oid);
    ($ok) = $cce->set($sys_oid, '',{ 'MN_BackupTrigger' => time() });
}

sub disable_backup {
    # Disable backup on the secondary:
    @oids = $cce->find('VserverMasterNode');
    if ($#oids < 0) {
        # Object not yet in CCE. Creating new one.
        ($ok) = $cce->create('VserverMasterNode', {
            'MN_LocalBackupEnabled' => "0",
            'MN_RemoteBackupEnabled' => "0"
        });
    }
    else {
        # Object already present in CCE. Updating it
        ($sys_oid) = $cce->find('VserverMasterNode');
        ($ok, $sys) = $cce->get($sys_oid);

        ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
            'MN_LocalBackupEnabled' => "0",
            'MN_RemoteBackupEnabled' => "0"
        });
    }
}

$cce->bye('SUCCESS');
exit(0);

sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

